// tutor
package repositories

import (
	"database/sql"
	"errors"
	"fmt"
	"log"
	"officeHours/models"
)

type TutorRepo struct {
	db *sql.DB
}

func NewTutorRepo(db *sql.DB) *TutorRepo {
	return &TutorRepo{
		db: db,
	}
}

func (r *TutorRepo) FindByEmail(email string) ([]models.Tutor, error) {
	rows, err := r.db.Query("SELECT * FROM tutor WHERE email=?", email)
	if err != nil {
		return nil, err
	}
	defer rows.Close()

	return r.getFromRows(rows)
}

func (r *TutorRepo) FindById(id int) (models.Tutor, error) {
	return r.getFromRow(r.db.QueryRow("SELECT * FROM tutor WHERE id=?", id))
}

func (r *TutorRepo) FindByNameAndEmail(name string, email string) (models.Tutor, error) {
	return r.getFromRow(r.db.QueryRow("SELECT * FROM tutor WHERE email=? AND name=?", email, name))
}

func (r *TutorRepo) GetAll() ([]models.Tutor, error) {
	rows, err := r.db.Query("SELECT * FROM tutor")
	if err != nil {
		return nil, err
	}
	defer rows.Close()

	return r.getFromRows(rows)
}
func (r *TutorRepo) Add(tutor models.Tutor) (int, error) {
	//Don't add identical tutors
	existentTutor, err := r.FindByNameAndEmail(tutor.Name, tutor.Email)
	if errors.Is(err, sql.ErrNoRows) {
		sqlResult, err := r.db.Exec("INSERT INTO `tutor` (name, email, subscribeToMailinglist) VALUES (?,?,?)", tutor.Name, tutor.Email, tutor.SubscribeToMailinglist)
		if err != nil {
			err = fmt.Errorf("SQL-error inserting new tutor: %w", err)
			log.Println(err.Error())
			return 0, err
		}
		id, lastInsertIdErr := sqlResult.LastInsertId()
		if lastInsertIdErr != nil {
			log.Printf("Error getting Id for new tutor: %s", lastInsertIdErr.Error())
		}
		return int(id), err
	}
	return existentTutor.Id, err
}

func (r *TutorRepo) getFromRow(row *sql.Row) (models.Tutor, error) {
	var tutor models.Tutor
	if err := row.Scan(&tutor.Id, &tutor.Name, &tutor.Email, &tutor.SubscribeToMailinglist); err != nil {
		err = fmt.Errorf("SQL-error scanning tutor row: %w", err)
		log.Println(err.Error())
		return models.Tutor{}, err
	}
	return tutor, nil
}

func (r *TutorRepo) getFromRows(rows *sql.Rows) ([]models.Tutor, error) {
	var tutors []models.Tutor
	for rows.Next() {
		var tutor models.Tutor
		if err := rows.Scan(&tutor.Id, &tutor.Name, &tutor.Email, &tutor.SubscribeToMailinglist); err != nil {
			err = fmt.Errorf("Error scanning tutor row: %w", err)
			log.Println(err.Error())
			return tutors, err
		}
		tutors = append(tutors, tutor)
	}
	return tutors, nil
}
